﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace MathSolver
{
    public partial class MainForm : Form
    {
        private List<MathProblem> _ProblemList;
        private OpenFileDialog _OpenFileDialog;

        public MainForm()
        {
            InitializeComponent();

            _ProblemList = new List<MathProblem>();
        }

        private void ButtonLoadData_Click(object sender, EventArgs e)
        {
            _OpenFileDialog = new OpenFileDialog();
            _OpenFileDialog.Filter = "Math Problems (*.txt)|*.txt";
            _OpenFileDialog.Title = "Open Math Problems";

            if (_OpenFileDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK) 
            {
                LoadData(_OpenFileDialog.FileName);
                FillData(false);
            }
        }

        private void ButtonSolve_Click(object sender, EventArgs e)
        {
            FillData(true);
        }

        private void LoadData(String fileName)
        {
            try
            {
                _ProblemList.Clear();
                StreamReader sr = new StreamReader(fileName);
                while (!sr.EndOfStream)
                {
                    String[] data = sr.ReadLine().Split(' ');
                    switch (data[0])
                    {
                        case "0":
                            _ProblemList.Add(new Factorial(Int32.Parse(data[1])));
                            break;
                        case "1":
                            _ProblemList.Add(new GreatestCommonDivisor(Int32.Parse(data[1]), Int32.Parse(data[2])));
                            break;
                        case "2":
                            _ProblemList.Add(new Fibonacci(Int32.Parse(data[1])));
                            break;
                    }

                }
                sr.Close();
            }
            catch (Exception)
            {
                MessageBox.Show("Error occured while loading data!");
            }
        }

        private void FillData(Boolean solve) 
        {
            _ListBox.Items.Clear();
            foreach(MathProblem problem in _ProblemList)
            {
                if (solve)
                    _ListBox.Items.Add(problem.Problem + " = " + problem.Solve());
                else
                    _ListBox.Items.Add(problem.Problem);
            }
        }
    }
}
