﻿using System;
using System.Collections.Generic;
using System.Text;

namespace MathSolver
{
    public abstract class MathProblem
    {
        public abstract String Problem { get; }
        public abstract Int32 Solve();
    }

    public class Factorial : MathProblem
    {
        private Int32 _Argument;

        public override String Problem
        {
            get { return "Fact(" + _Argument + ")"; }
        }

        public Factorial(Int32 n)
        {
            if (n <= 0)
                throw new ArgumentException();
            _Argument = n;
        }

        public override Int32 Solve()
        {
            return F(_Argument);
        }

        private Int32 F(Int32 n)
        {
            if (n > 1)
                return n * F(n - 1);
            else
                return 1;
        }
    }

    public class Fibonacci : MathProblem
    {
        private Int32 _Argument;

        public override String Problem
        {
            get { return "Fibonacci(" + _Argument + ")"; }
        }

        public Fibonacci(Int32 n)
        {
            if (n <= 0)
                throw new ArgumentException();
            _Argument = n;
        }

        public override Int32 Solve()
        {
            return Fib(_Argument);
        }

        private Int32 Fib(Int32 n) 
        {
            if (n > 2)
                return Fib(n - 1) + Fib(n - 2);
            else
                return 1;
        }
    }

    public class GreatestCommonDivisor : MathProblem
    {
        private Int32 _Argument1, _Argument2;

        public override String Problem
        {
            get { return "GCD(" + _Argument1 + ", " + _Argument2 + ")"; }
        }

        public GreatestCommonDivisor(Int32 n, Int32 m)
        {
            if (n <= 0 || m <= 0)
                throw new ArgumentException();
            _Argument1 = n;
            _Argument2 = m;
        }

        public override Int32 Solve()
        {
            Int32 x = _Argument1, y = _Argument2;
            while (x != y)
            {
                if (x < y)
                    y -= x;
                else
                    x -= y;
            }
            return x;
        }
    }
}
